package lols.core.client.android;

import com.thoughtworks.xstream.XStream;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpOutputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.AbstractHttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.converter.HttpMessageNotWritableException;

import java.io.*;
import java.nio.charset.Charset;

public class XStreamHttpMessageConverter extends AbstractHttpMessageConverter<Object> {

    public static final Charset DEFAULT_CHARSET = Charset.forName("UTF-8");

    private final XStream xStream;

    public XStreamHttpMessageConverter(XStream xStream) {
        super(MediaType.APPLICATION_XML, MediaType.TEXT_XML, MediaType.APPLICATION_WILDCARD_XML);
        this.xStream = xStream;
    }

    public XStreamHttpMessageConverter() {
        this(new XStream());
    }

    protected boolean supports(Class<?> aClass) {
        return true;
    }

    protected Object readInternal(Class aClass, HttpInputMessage inputMessage) throws IOException, HttpMessageNotReadableException {
        Reader source = new InputStreamReader(inputMessage.getBody(), getCharset(inputMessage.getHeaders()));

        try {
            return xStream.fromXML(source);
        } catch (Exception ex) {
            throw new HttpMessageNotReadableException("Could not read XML.", ex);
        }
    }

    protected void writeInternal(Object o, HttpOutputMessage outputMessage) throws IOException, HttpMessageNotWritableException {
        Writer out = new OutputStreamWriter(outputMessage.getBody(), getCharset(outputMessage.getHeaders()));

        try {
            this.xStream.toXML(o, out);
            out.close();
        } catch (Exception ex) {
            throw new HttpMessageNotWritableException("Could not write [" + o + "]", ex);
        }
    }

    private Charset getCharset(HttpHeaders headers) {
        if (headers != null && headers.getContentType() != null
                && headers.getContentType().getCharSet() != null) {
            return headers.getContentType().getCharSet();
        }
        return DEFAULT_CHARSET;
    }

}
